from PyQt5.QtGui import QPixmap, QImage         
from PyQt5.QtWidgets import QWidget,QMainWindow, QLabel, QSizePolicy, QApplication, QAction, QHBoxLayout,QProgressBar
from PyQt5.QtCore import Qt,QEvent,QObject
from PyQt5.QtCore import *
import sys,traceback

import ctypes as C
import numpy as np
import cv2

# PyhtonNetをインポートする
import clr
# IC Imaging Control3.5を参照する
clr.AddReference('TIS.Imaging.ICImagingControl35')


# IC Imaging Control namespaceを宣言
import TIS.Imaging

class SinkData:
    brightnes = 0
    FrameBuffer = None


class DisplayBuffer:
    '''
    ビデオウィンドウに表示するために画像をpixmapにコピーするために必要
    '''
    locked = False
    pixmap = None

    def Copy( self, FrameBuffer):
        if(  int(FrameBuffer.FrameType.BitsPerPixel/8 ) == 4):
            imgcontent = C.cast(FrameBuffer.GetIntPtr().ToInt64(), C.POINTER(C.c_ubyte * FrameBuffer.FrameType.BufferSize))
            qimage = QImage(imgcontent.contents, FrameBuffer.FrameType.Width,FrameBuffer.FrameType.Height, QImage.Format_RGB32).mirrored()
            self.pixmap = QPixmap(qimage)

class WorkerSignals(QObject):
    display = pyqtSignal(object)
    result = pyqtSignal(object)



class DisplayFilter(TIS.Imaging.FrameFilterImpl):
    '''
    到着フレームをDisplayBufferオブジェクトにコピーし、新しいバッファーのQApplicationに送る
    '''
    __namespace__ = "DisplayFilterClass"
    signals = WorkerSignals() 
    dispBuffer = DisplayBuffer()
    
    def GetSupportedInputTypes(self, frameTypes):
        frameTypes.Add( TIS.Imaging.FrameType(TIS.Imaging.MediaSubtypes.RGB32))

    def GetTransformOutputTypes(self,inType, outTypes):
        outTypes.Add(inType)
        return True

    def Transform(self, src, dest):
        dest.CopyFrom(src)
        if self.dispBuffer.locked is False:
            self.dispBuffer.locked = True
            self.dispBuffer.Copy(dest)
            self.signals.display.emit(self.dispBuffer)

        return False



# イメージバッファを受け取ったときに起動するクラス（イベントリスナー）
class Listener(TIS.Imaging.IFrameQueueSinkListener):
    __namespace__ = 'ListenerNameSpace'

    saveimage=False
    signals = WorkerSignals() 
    
    #シンク（画像バッファ）に接続したときに発火
    def SinkConnected(self, sink, frameType ):
        print('Sink connected')
        #バッファリストに割り当て、キューに割り当てる
        sink.AllocAndQueueBuffers(2)

    #シンク（画像バッファ）から接続を外したときに発火
    def SinkDisconnected(self, sink,buffers):
        print('Sink disconnected')


    # デバイスからフレームを受け取った時に発火
    def FramesQueued(self,  sink):
        frame = sink.PopOutputQueueBuffer()
        if self.saveimage is True:
            self.saveimage = False
            TIS.Imaging.FrameExtensions.SaveAsJpeg(frame,"test.jpg",75)

        data = SinkData()
        data.FrameBuffer = frame
        self.signals.result.emit(data)


def SelectDevice():
    ic.LiveStop()
    ic.ShowDeviceSettingsDialog()
    if ic.DeviceValid is True:
        ic.LiveStart()
        ic.SaveDeviceStateToFile("device.xml")

def ShowProperties():
    if ic.DeviceValid is True:
        ic.ShowPropertyDialog()
        ic.SaveDeviceStateToFile("device.xml")

def SnapImage():
    listener.saveimage = True
    print("Snap")

def Close():
    if ic.DeviceValid is True:
        ic.LiveStop()
    app.quit()

def OnResult(result):
  
    imgcontent = C.cast(result.FrameBuffer.GetIntPtr().ToInt64(), C.POINTER(C.c_ubyte * result.FrameBuffer.FrameType.BufferSize))
    img = np.ndarray(buffer = imgcontent.contents,
                    dtype = np.uint8,
                    shape = (result.FrameBuffer.FrameType.Height,
                            result.FrameBuffer.FrameType.Width,
                            int(result.FrameBuffer.FrameType.BitsPerPixel/8 )) )
    
    # 平均輝度値の平均化
    gray = cv2.cvtColor(img ,cv2.COLOR_BGR2GRAY)
    mean = cv2.mean(gray)
    brightnessbar.setValue(int(mean[0]))

    sink.QueueBuffer(result.FrameBuffer)

def OnDisplay(dispBuffer):
    videowindow.setPixmap(dispBuffer.pixmap)   
    dispBuffer.locked = False   

app =  QApplication(sys.argv)

w = QMainWindow()
w.resize(640, 480)
w.move(300, 300)
w.setWindowTitle('imageprocessing Camera')

# メニュー作成
mainMenu = w.menuBar()
fileMenu = mainMenu.addMenu('&ファイル')

exitAct =  QAction("&終了",app)
exitAct.setStatusTip("Exit program")
exitAct.triggered.connect(Close)
fileMenu.addAction(exitAct)

deviceMenu = mainMenu.addMenu('&デバイス')
devselAct =  QAction("&選択",app)
devselAct.triggered.connect(SelectDevice)
deviceMenu.addAction(devselAct)

devpropAct =  QAction("&プロパティ",app)
devpropAct.triggered.connect(ShowProperties)
deviceMenu.addAction(devpropAct)

snapAct =  QAction("bmpで保存",app)
snapAct.triggered.connect(SnapImage)
deviceMenu.addAction(snapAct)

layout = QHBoxLayout()
mainwindow = QWidget()
videowindow = QLabel()
layout.addWidget(videowindow)

brightnessbar = QProgressBar()
brightnessbar.setRange(0,256)
brightnessbar.setOrientation( Qt.Vertical)
brightnessbar.setValue(25)
layout.addWidget(brightnessbar)

mainwindow.setLayout(layout)
w.setCentralWidget(mainwindow)

################################################
#IC ImagingControlの設定

# IC ImagingControlオブジェクトを作成
ic = TIS.Imaging.ICImagingControl()

# ライブ表示用に表示フィルターオブジェクトをインスタンス化します
displayFilter = DisplayFilter()

# ディスプレイ信号ハンドラーをフィルターに接続します
displayFilter.signals.display.connect(OnDisplay)
ic.DisplayFrameFilters.Add( ic.FrameFilterCreate(displayFilter))

# 画像処理するためのイベントリスナーを作成
listener = Listener()
sink = TIS.Imaging.FrameQueueSink(listener,TIS.Imaging.MediaSubtypes.RGB32)

ic.Sink = sink

# リスナーにシグナルを接続
listener.signals.result.connect(OnResult)

ic.LiveDisplay = True


try:
    ic.LoadDeviceStateFromFile("device.xml",True)
    if ic.DeviceValid is True:
        ic.LiveStart()
except Exception as ex:
    print(ex)
    pass

w.show()

app.exec()
